const css = require('css');

module.exports = function px2rem(code, options = {}) {
  const { compress } = options;
  const rem = options.rem || 16;
  const dpr = options.dpr || 1;
  const remValue = rem * dpr;

  const ast = css.parse(code);
  const rules = ast.stylesheet.rules;

  const ignoreLines = getIgnoreLines(rules);

  calculateREM(rules, remValue, ignoreLines);

  const result = css.stringify(ast, { compress });
  return result;
}

function calculateREM(rules, remValue, ignoreLines) {
  rules && rules.forEach(rule => {
    rule.declarations && rule.declarations.forEach(declaration => {
      const { property, value, position } = declaration;
      const isPxValue = property
        && !property.match(/border/)
        && value.match(/px/)
        && ignoreLines.indexOf(position.start.line) === -1;

      if (isPxValue) {
        declaration.value = convertPxValueToRem(value, remValue);
      }
    })
  })
}

function convertPxValueToRem(value, remValue) {
  return value.replace(/-?\d*px/g, pxAttr =>{
    const pxValue = pxAttr.replace('px', '');
    return parseFloat(pxValue / remValue).toFixed(3) + 'rem';
  });
}

function getIgnoreLines(rules) {
  const ignoreLines = [];

  rules && rules.forEach(rule => {
    rule.declarations && rule.declarations
      .forEach(({ type, comment, position }, index) => {
        if (type === 'comment' && comment === 'px') {
          ignoreLines.push(position.start.line);
          rule.declarations.splice(index, 1);
        }
      })
  })

  return ignoreLines;
}

